@@ -5,7 +5,7 @@ use std::marker::PhantomData;
 macro_rules! impl_char_parser {
     ($name: ident ($($ty_var: ident),*), $inner_type: ty) => {
     #[derive(Clone)]
-    pub struct $name<I $(,$ty_var)*>($inner_type, PhantomData<I>)
+    pub struct $name<I $(,$ty_var)*>($inner_type, PhantomData<fn (I) -> I>)
         where I: Stream<Item=char> $(, $ty_var : Parser<Input=I>)*;
     impl <I $(,$ty_var)*> Parser for $name<I $(,$ty_var)*>
         where I: Stream<Item=char> $(, $ty_var : Parser<Input=I>)* {
@@ -18,12 +18,35 @@ macro_rules! impl_char_parser {
 }
 }
 
-///Parses any character
-pub fn any_char<I>(input: State<I>) -> ParseResult<char, I>
+#[derive(Clone)]
+pub struct AnyChar<I>(PhantomData<fn (I) -> I>);
+
+impl <I> Parser for AnyChar<I>
     where I: Stream<Item=char> {
-    input.uncons_char()
+    type Input = I;
+    type Output = char;
+    fn parse_state(&mut self, input: State<I>) -> ParseResult<char, I> {
+        input.uncons_char()
+    }
 }
 
+///Parses any character
+///
+/// ```
+/// # extern crate parser_combinators as pc;
+/// # use pc::*;
+/// # fn main() {
+/// let mut parser = any_char();
+/// assert_eq!(parser.parse("!").map(|x| x.0), Ok('!'));
+/// assert!(parser.parse("").is_err());
+/// # }
+/// ```
+pub fn any_char<I>() -> AnyChar<I>
+    where I: Stream<Item=char> {
+    AnyChar(PhantomData)
+}
+
+
 #[derive(Clone)]
 pub struct Satisfy<I, Pred> { pred: Pred, _marker: PhantomData<I> }
 
@@ -51,10 +74,9 @@ impl <I, Pred> Parser for Satisfy<I, Pred>
 /// # extern crate parser_combinators as pc;
 /// # use pc::*;
 /// # fn main() {
-/// let result = satisfy(|c| c == '!')
-///     .parse("!")
-///     .map(|x| x.0);
-/// assert_eq!(result, Ok('!'));
+/// let mut parser = satisfy(|c| c == '!' || c == '?');
+/// assert_eq!(parser.parse("!").map(|x| x.0), Ok('!'));
+/// assert_eq!(parser.parse("?").map(|x| x.0), Ok('?'));
 /// # }
 /// ```
 pub fn satisfy<I, Pred>(pred: Pred) -> Satisfy<I, Pred>
@@ -62,6 +84,45 @@ pub fn satisfy<I, Pred>(pred: Pred) -> Satisfy<I, Pred>
     Satisfy { pred: pred, _marker: PhantomData }
 }
 
+#[derive(Clone)]
+pub struct Char<I> { c: char, _marker: PhantomData<I> }
+
+impl <I> Parser for Char<I>
+    where I: Stream<Item=char> {
+
+    type Input = I;
+    type Output = char;
+    fn parse_state(&mut self, input: State<I>) -> ParseResult<char, I> {
+        match input.clone().uncons_char() {
+            Ok((c, s)) => {
+                if self.c == c { Ok((c, s)) }
+                else {
+                    let errors = vec![Error::Unexpected(c), Error::Expected(self.c.into())];
+                    Err(Consumed::Empty(ParseError::from_errors(input.position, errors)))
+                }
+            }
+            Err(err) => Err(err)
+        }
+    }
+}
+
+///Parses a character and succeeds if the characther is equal to `c`
+///
+/// ```
+/// # extern crate parser_combinators as pc;
+/// # use pc::*;
+/// # fn main() {
+/// let result = char('!')
+///     .parse("!")
+///     .map(|x| x.0);
+/// assert_eq!(result, Ok('!'));
+/// # }
+/// ```
+pub fn char<I>(c: char) -> Char<I>
+    where I: Stream<Item=char> {
+    Char { c: c, _marker: PhantomData }
+}
+
 impl_char_parser! { Digit(), Expected<Satisfy<I, fn (char) -> bool>> }
 ///Parses a digit from a stream containing characters
 pub fn digit<I>() -> Digit<I>
@@ -173,7 +234,8 @@ impl <I> Parser for String<I>
             match input.uncons_char() {
                 Ok((other, rest)) => {
                     if c != other {
-                        let error = ParseError::new(start, Error::Expected(self.0.into()));
+                        let errors = vec![Error::Unexpected(other), Error::Expected(self.0.into())];
+                        let error = ParseError::from_errors(start, errors);
                         return Err(if consumed { Consumed::Consumed(error) } else { Consumed::Empty(error) })
                     }
                     consumed = true;
@@ -212,7 +274,7 @@ pub fn string<I>(s: &'static str) -> String<I>
 #[cfg(test)]
 mod tests {
     use super::*;
-    use primitives::{Error, Parser, SourcePosition};
+    use primitives::{Error, ParseError, Parser, SourcePosition};
 
     #[test]
     fn space_error() {
@@ -229,4 +291,13 @@ mod tests {
         assert!(result.is_err());
         assert_eq!(result.unwrap_err().position, SourcePosition { line: 1, column: 1 });
     }
+
+    #[test]
+    fn string_error() {
+        let result = string("abc").parse("bc");
+        assert_eq!(result, Err(ParseError {
+            position: SourcePosition { line: 1, column: 1 },
+            errors: vec![Error::Unexpected('b'), Error::Expected("abc".into())]
+        }));
+    }
 }
