@@ -1,7 +1,6 @@
 use std::iter::FromIterator;
-use std::borrow::Cow;
 use std::marker::PhantomData;
-use primitives::{Parser, ParseResult, ParseError, Stream, State, Error, Consumed};
+use primitives::{Info, Parser, ParseResult, ParseError, Stream, State, Error, Consumed};
 
 macro_rules! impl_parser {
     ($name: ident ($first: ident, $($ty_var: ident),*), $inner_type: ty) => {
@@ -69,7 +68,7 @@ pub fn choice<S, P>(ps: S) -> Choice<S, P>
 }
 
 #[derive(Clone)]
-pub struct Unexpected<I>(Cow<'static, str>, PhantomData<I>);
+pub struct Unexpected<I>(Info, PhantomData<fn (I) -> I>);
 impl <I> Parser for Unexpected<I>
     where I : Stream {
     type Input = I;
@@ -94,12 +93,12 @@ impl <I> Parser for Unexpected<I>
 /// ```
 pub fn unexpected<I, S>(message: S) -> Unexpected<I>
     where I: Stream
-        , S: Into<Cow<'static, str>> {
+        , S: Into<Info> {
     Unexpected(message.into(), PhantomData)
 }
 
 #[derive(Clone)]
-pub struct Value<I, T>(T, PhantomData<I>);
+pub struct Value<I, T>(T, PhantomData<fn (I) -> I>);
 impl <I, T> Parser for Value<I, T>
     where I: Stream
         , T: Clone {
@@ -136,7 +135,7 @@ impl_parser! { NotFollowedBy(P,), Or<Then<Try<P>, fn(<P as Parser>::Output) -> U
 /// # use pc::*;
 /// # fn main() {
 /// let result = string("let")
-///     .skip(not_followed_by(satisfy(|c| c.is_alphanumeric())))
+///     .skip(not_followed_by(alpha_num()))
 ///     .parse("letx")
 ///     .map(|x| x.0);
 /// assert!(result.is_err());
@@ -227,7 +226,7 @@ pub fn many<F, P>(p: P) -> Many<F, P>
 
 
 #[derive(Clone)]
-pub struct Many1<F, P>(P, PhantomData<F>);
+pub struct Many1<F, P>(P, PhantomData<fn () -> F>);
 impl <F, P> Parser for Many1<F, P>
     where F: FromIterator<<P as Parser>::Output>
         , P: Parser {
@@ -310,7 +309,7 @@ pub fn many1<F, P>(p: P) -> Many1<F, P>
 pub struct SepBy<F, P, S> {
     parser: P,
     separator: S,
-    _marker: PhantomData<F>
+    _marker: PhantomData<fn () -> F>
 }
 impl <F, P, S> Parser for SepBy<F, P, S>
     where F: FromIterator<<P as Parser>::Output>
@@ -353,7 +352,7 @@ impl <F, P, S> Parser for SepBy<F, P, S>
 /// # extern crate parser_combinators as pc;
 /// # use pc::*;
 /// # fn main() {
-/// let result = sep_by(digit(), satisfy(|c| c == ','))
+/// let result = sep_by(digit(), char(','))
 ///     .parse("1,2,3")
 ///     .map(|x| x.0);
 /// assert_eq!(result, Ok(vec!['1', '2', '3']));
@@ -375,7 +374,7 @@ impl <'a, I: Stream, O> Parser for FnMut(State<I>) -> ParseResult<O, I> + 'a {
     }
 }
 #[derive(Clone)]
-pub struct FnParser<I, F>(F, PhantomData<fn (I)>);
+pub struct FnParser<I, F>(F, PhantomData<fn (I) -> I>);
 
 ///Wraps a function, turning it into a parser
 ///Mainly needed to turn closures into parsers as function types can be casted to function pointers
@@ -468,7 +467,7 @@ impl_parser! { Between(L, R, P), Skip<With<L, P>, R> }
 /// # extern crate parser_combinators as pc;
 /// # use pc::*;
 /// # fn main() {
-/// let result = between(string("["), string("]"), string("rust"))
+/// let result = between(char('['), char(']'), string("rust"))
 ///     .parse("[rust]")
 ///     .map(|x| x.0);
 /// assert_eq!(result, Ok("rust"));
@@ -484,10 +483,11 @@ pub fn between<I, L, R, P>(open: L, close: R, parser: P) -> Between<L, R, P>
 
 #[derive(Clone)]
 pub struct Chainl1<P, Op>(P, Op);
-impl <'a, I, O, P, Op> Parser for Chainl1<P, Op>
+impl <I, O, P, Op> Parser for Chainl1<P, Op>
     where I: Stream
         , P: Parser<Input=I, Output=O>
-        , Op: Parser<Input=I, Output=Box<FnMut(O, O) -> O + 'a>> {
+        , Op: Parser<Input=I>
+        , Op::Output: FnOnce(O, O) -> O {
 
     type Input = I;
     type Output = O;
@@ -497,7 +497,7 @@ impl <'a, I, O, P, Op> Parser for Chainl1<P, Op>
             let was_empty = input.is_empty();
             let rest = input.clone().into_inner();
             match (&mut self.1).and(&mut self.0).parse_state(rest) {
-                Ok(((mut op, r), rest)) => {
+                Ok(((op, r), rest)) => {
                     l = op(l, r);
                     input = if was_empty { rest } else { rest.as_consumed() };
                 }
@@ -513,18 +513,20 @@ impl <'a, I, O, P, Op> Parser for Chainl1<P, Op>
 
 ///Parses `p` 1 or more times separated by `op`
 ///The value returned is the one produced by the left associative application of `op`
-pub fn chainl1<'a, P, Op>(parser: P, op: Op) -> Chainl1<P, Op>
+pub fn chainl1<P, Op>(parser: P, op: Op) -> Chainl1<P, Op>
     where P: Parser
-        , Op: Parser<Input=<P as Parser>::Input, Output=Box<FnMut(<P as Parser>::Output, <P as Parser>::Output) -> <P as Parser>::Output + 'a>> {
+        , Op: Parser<Input=P::Input>
+        , Op::Output: FnOnce(P::Output, P::Output) -> P::Output {
     Chainl1(parser, op)
 }
 
 #[derive(Clone)]
 pub struct Chainr1<P, Op>(P, Op);
-impl <'a, I, O, P, Op> Parser for Chainr1<P, Op>
+impl <I, O, P, Op> Parser for Chainr1<P, Op>
     where I: Stream
         , P: Parser<Input=I, Output=O>
-        , Op: Parser<Input=I, Output=Box<FnMut(O, O) -> O + 'a>> {
+        , Op: Parser<Input=I>
+        , Op::Output: FnOnce(O, O) -> O {
 
     type Input = I;
     type Output = O;
@@ -533,7 +535,7 @@ impl <'a, I, O, P, Op> Parser for Chainr1<P, Op>
         loop {
             let was_empty = input.is_empty();
             let rest = input.clone().into_inner();
-            let mut op = match self.1.parse_state(rest) {
+            let op = match self.1.parse_state(rest) {
                 Ok((x, rest)) => {
                     input = if was_empty { rest } else { rest.as_consumed() };
                     x
@@ -560,9 +562,10 @@ impl <'a, I, O, P, Op> Parser for Chainr1<P, Op>
 
 ///Parses `p` one or more times separated by `op`
 ///The value returned is the one produced by the right associative application of `op`
-pub fn chainr1<'a, P, Op>(parser: P, op: Op) -> Chainr1<P, Op>
+pub fn chainr1<P, Op>(parser: P, op: Op) -> Chainr1<P, Op>
     where P: Parser
-        , Op: Parser<Input=<P as Parser>::Input, Output=Box<FnMut(<P as Parser>::Output, <P as Parser>::Output) -> <P as Parser>::Output + 'a>> {
+        , Op: Parser<Input=P::Input>
+        , Op::Output: FnOnce(P::Output, P::Output) -> P::Output {
     Chainr1(parser, op)
 }
 
@@ -641,7 +644,7 @@ impl <I, P1, P2> Parser for Skip<P1, P2>
 }
 
 #[derive(Clone)]
-pub struct Message<P>(P, Cow<'static, str>) where P: Parser;
+pub struct Message<P>(P, Info) where P: Parser;
 impl <I, P> Parser for Message<P>
     where I: Stream, P: Parser<Input=I> {
 
@@ -715,7 +718,7 @@ impl <P, N, F> Parser for Then<P, F>
 }
 
 #[derive(Clone)]
-pub struct Expected<P>(P, Cow<'static, str>);
+pub struct Expected<P>(P, Info);
 impl <P> Parser for Expected<P>
     where P: Parser {
 
@@ -763,7 +766,7 @@ pub trait ParserExt : Parser + Sized {
     /// # use pc::*;
     /// # fn main() {
     /// let result = digit()
-    ///     .with(satisfy(|c| c == 'i'))
+    ///     .with(char('i'))
     ///     .parse("9i")
     ///     .map(|x| x.0);
     /// assert_eq!(result, Ok('i'));
@@ -782,7 +785,7 @@ pub trait ParserExt : Parser + Sized {
     /// # use pc::*;
     /// # fn main() {
     /// let result = digit()
-    ///     .skip(satisfy(|c| c == 'i'))
+    ///     .skip(char('i'))
     ///     .parse("9i")
     ///     .map(|x| x.0);
     /// assert_eq!(result, Ok('9'));
@@ -802,7 +805,7 @@ pub trait ParserExt : Parser + Sized {
     /// # use pc::*;
     /// # fn main() {
     /// let result = digit()
-    ///     .and(satisfy(|c| c == 'i'))
+    ///     .and(char('i'))
     ///     .parse("9i")
     ///     .map(|x| x.0);
     /// assert_eq!(result, Ok(('9', 'i')));
@@ -883,7 +886,7 @@ pub trait ParserExt : Parser + Sized {
     /// # use pc::*;
     /// # use pc::primitives::Error;
     /// # fn main() {
-    /// let result = satisfy(|c| c == '9')
+    /// let result = char('9')
     ///     .message("Not a nine")
     ///     .parse("8");
     /// assert!(result.is_err());
@@ -892,7 +895,7 @@ pub trait ParserExt : Parser + Sized {
     /// # }
     /// ```
     fn message<S>(self, msg: S) -> Message<Self>
-        where S: Into<Cow<'static, str>> {
+        where S: Into<Info> {
         Message(self, msg.into())
     }
 
@@ -904,7 +907,7 @@ pub trait ParserExt : Parser + Sized {
     /// # use pc::*;
     /// # use pc::primitives::Error;
     /// # fn main() {
-    /// let result = satisfy(|c| c == '9')
+    /// let result = char('9')
     ///     .expected("9")
     ///     .parse("8");
     /// assert!(result.is_err());
@@ -913,7 +916,7 @@ pub trait ParserExt : Parser + Sized {
     /// # }
     /// ```
     fn expected<S>(self, msg: S) -> Expected<Self>
-        where S: Into<Cow<'static, str>> {
+        where S: Into<Info> {
         Expected(self, msg.into())
     }
 
@@ -943,12 +946,12 @@ impl <P: Parser> ParserExt for P { }
 mod tests {
     use super::*;
     use primitives::Parser;
-    use char::{digit, string};
+    use char::{char, digit};
 
     #[test]
     fn chainr1_test() {
         let number = digit().map(|c| c.to_digit(10).unwrap() as i32);
-        let pow = string("^").map(|_| { let f: Box<FnMut(_, _) -> _> = Box::new(|l:i32, r:i32| l.pow(r as u32)); f });
+        let pow = char('^').map(|_| |l:i32, r:i32| l.pow(r as u32));
         let mut parser = chainr1(number, pow);
         assert_eq!(parser.parse("2^3^2"), Ok((512, "")));
     }
